<?php
/**
 *
 * Allow a campaign to be triggered via a json endpoint
 * REQUIRES THAT ZENGERFEED_IMPORT_KEY be defined in your wp-config.php file
 * Example URI:
 * yoursite.com/wp-json/v1/import?key=[ZENGERFEED_IMPORT_KEY]&campaign_id=[campaign ID from edit screen]
 *
 */


add_action( 'rest_api_init', 'zengerfeed_init_interface' );

function zengerfeed_init_interface() {

    register_rest_route( 'v1', 'import', array(
            'methods' => [ 'GET', 'POST' ],
            'callback' => 'zengerfeed_initialize_import',
        )

    );
}

/**
 * @param WP_REST_Request $request
 * @return mixed|WP_Error|WP_HTTP_Response|WP_REST_Response
 */
function zengerfeed_initialize_import( WP_REST_Request $request )
{
    if( !defined( 'ZENGERFEED_IMPORT_KEY' ) )
    {
        return rest_ensure_response( [
            'message' => 'Invalid request [1]',
        ] );
    }
    if( empty( $request->get_param( 'key' ) ) || $request->get_param( 'key' ) !== ZENGERFEED_IMPORT_KEY )
    {
        return rest_ensure_response( [
            'message' => 'Invalid request [2]',
        ] );
    }

    if( empty( $request->get_param( 'campaign_id' ) ) )
    {
        return rest_ensure_response( [
            'message' => 'Invalid campaign ID [3]',
        ] );
    }

    $campaign_id = absint( $request->get_param( 'campaign_id' ) );
    $response = substr( WPeMatico::wpematico_dojob( $campaign_id ) , 0, 60 );

    return rest_ensure_response( [
        'success' => true,
        'campaign_id' => $request->get_param( 'campaign_id' ),
        'time' => time(),
        'response' => trim( strip_tags( $response ) ),
    ] );
}