<?php
/**
 * Zenger Feed plugin for WordPress
 * plugin_functions
 * Contains all the hooks to be called for the plugins wordpress page and the activate/deactivate/uninstall methods.

 * @package   wpematico
 * @author    Zenger News
 * @copyright 2020 Zenger News
 * @license   GPL v2 or later
 */

// don't load directly 
if ( !defined('ABSPATH') ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit();
}

add_filter(	'plugin_row_meta', 'wpematico_row_meta',10,2);
add_filter(	'plugin_action_links_' . WPEMATICO_BASENAME, 'wpematico_action_links');
add_action( "after_plugin_row_" . WPEMATICO_BASENAME, 'wpematico_update_row', 10, 2 );
add_action( 'admin_head', 'WPeMatico_plugins_admin_head' );

function WPeMatico_plugins_admin_head(){
	global $pagenow, $page_hook;
	if($pagenow=='plugins.php'){
	?>
	<script type="text/javascript">
		jQuery(document).ready(function($){
			$('tr[data-slug=wpematico]').addClass('update');
		});
	</script>
	<style type="text/css">
		@media screen and (max-width: 782px) {
			#wpematico_addons_row{
			      display: none;
			}
		}
		.wpematico_active_addon {
			padding: 1px 5px;
			background: orange;
			border-radius: 5px;
			margin: 0 3px;
}
		.wpematico_addon {
			padding: 1px 5px;
			background: #8f7444;
			border-radius: 5px;
			margin: 0 3px;
}
	</style>
	<?php 
	}
}


/**
* Actions-Links del Plugin
*
* @param   array   $data  Original Links
* @return  array   $data  modified Links
*/
function wpematico_update_row( $file, $plugin_data ) {
	$plugins = get_plugins();
	$addons=read_wpem_addons( $plugins );
	$installed = "";
	foreach($addons as $key => $plugin) {
		if( !isset($plugin['installed'])) {
			unset( $addons[ $key ] );
		}elseif( $plugin['installed']) {
			if(is_plugin_active($key)){
				$installed .= '<span class="wpematico_active_addon">'.str_replace("WPeMatico", "",$plugin['Name']).'</span> ';
			}else{
				$installed .= '<span class="wpematico_addon">'.str_replace("WPeMatico", "",$plugin['Name']).'</span> ';
			}
		}
	}

}

/**
* Actions-Links del Plugin
*
* @param   array   $data  Original Links
* @return  array   $data  modified Links
*/
function wpematico_action_links($data)	{
	if ( !current_user_can('manage_options') ) {
		return $data;
	}
	return array_merge(	$data,	array(
		'<a href="edit.php?post_type=wpematico&page=wpematico_settings" title="' . __('Load WPeMatico Settings Page', 'wpematico' ) . '">' . __('Settings', 'wpematico' ) . '</a>',
//		'<a href="https://etruel.com/checkout?edd_action=add_to_cart&download_id=4313&edd_options[price_id]=2" target="_Blank" title="' . __('Buy all bundled Addons', 'wpematico' ) . '">' . __('Go Perfect', 'wpematico' ) . '</a>',
	));
}

/**
* Meta-Links del Plugin
*
* @param   array   $data  Original Links
* @param   string  $page  plugin actual
* @return  array   $data  modified Links
*/

function wpematico_row_meta($data, $page)	{
	if ( $page != WPEMATICO_BASENAME ) {
		return $data;
	}

	return array_merge(	$data,	array(
		//'<a href="http://www.wpematico.com/wpematico/" target="_blank">' . __('Info & comments') . '</a>',
//		'<p>' . __('Activated Extensions:', 'wpematico' ) . '</p>',
	));
}		



/***************************************************************************************
/***************************************************************************************
 * Activation, Upgrading and uninstall functions
 **************************************************************************************/
register_activation_hook( WPEMATICO_BASENAME, 'wpematico_activate' );
register_deactivation_hook( WPEMATICO_BASENAME, 'wpematico_deactivate' );
register_uninstall_hook( WPEMATICO_BASENAME, 'wpematico_uninstall' );

add_action( 'plugins_loaded', 'wpematico_update_db_check' );

function wpematico_update_db_check() {
	if (version_compare(WPEMATICO_VERSION, get_option( 'wpematico_db_version' ), '>')) { // check if updated (WILL SAVE new version on welcome )
		if ( !get_transient( '_wpematico_activation_redirect' ) ){ //just one time running
	        wpematico_install( false );  // true will re-save all the campaigns 
		}
    }
}


function send_activated_email()
{
	$headers[] = 'From: '.get_bloginfo( 'name' ).' <'. get_bloginfo( 'admin_email' ) .'>';
	$message = get_bloginfo( 'name' ) . ' at ' .get_bloginfo( 'url' ) .' installed the publication at '. date( 'Y-m-d h:i a' );
	wp_mail( 'andre@zenger.news', get_bloginfo( 'name' ) .' has signed up for syndication', $message, $headers );
}

function set_base_campaigns()
{
	//If the plugin was already installed, don't redo all this
	$exists = get_option( 'wpematico_db_version' );
	if( $exists != '' )
	{
		return;
	}

	//Create the zenger news user for these posts
	$user = get_user_by( 'email', 'info@zenger.news' );
	if( !is_null( $user ) && isset( $user->ID ) )
	{
		$zenger_user = $user->ID;
	}
	else
	{
		$userdata = [
			'user_login' => 'zenger.news',
			'user_nicename' => 'zenger.news',
			'user_email' => 'info@zenger.news',
			'user_pass' => wp_generate_password(),
			'user_url' => 'https://zenger.news',
			'role' => 'subscriber',
		];
		$zenger_user = wp_insert_user( $userdata );
	}
	/*
	 * All Content: https://zenger.news/rss2

ADDED BUT PAUSED:
	Urban News: https://zenger.news/category/urban-news/rss2
	World News: https://zenger.news/category/world/rss2
	https://zenger.news/category/politics/rss2
	https://zenger.news/category/business/rss2
	https://zenger.news/category/hispanic-news/rss2
	https://zenger.news/category/video/rss2
	https://zenger.news/category/health/rss2
	 */
	$campaigns = [
		[
			'title' => 'All Content',
			'link' => 'https://zenger.news/rss2',
		],
		[
			'title' => 'Urban News',
			'link' => 'https://zenger.news/category/urban-news/rss2',
			'cron' => '5 * * * *',
		],
		[
			'title' => 'World News',
			'link' => 'https://zenger.news/category/world/rss2',
			'cron' => '10 * * * *',
		],
		[
			'title' => 'Politics',
			'link' => 'https://zenger.news/category/politics/rss2',
			'cron' => '15 * * * *',
		],
		[
			'title' => 'Business',
			'link' => 'https://zenger.news/category/business/rss2',
			'cron' => '20 * * * *',
		],
		[
			'title' => 'Hispanic News',
			'link' => 'https://zenger.news/category/hispanic/rss2',
			'cron' => '25 * * * *',
		],
		[
			'title' => 'Video',
			'link' => 'https://zenger.news/category/video/rss2',
			'cron' => '30 * * * *',
		],
		[
			'title' => 'Health',
			'link' => 'https://zenger.news/category/health/rss2',
			'cron' => '35 * * * *',
		],
        [
            'title' => 'Español',
            'link' => 'https://www.zenger.news/category/es/rss2',
            'cron' => '40 * * * *',
        ],
	];

	foreach( $campaigns as $campaign )
	{
		$item = [
			'post_type' => 'wpematico',
			'post_title' => $campaign['title'],
			'post_content' => '',
			'post_status' => isset( $campaign['published'] ) ? 'publish' : 'draft',
		];
		$post_id = wp_insert_post( $item );

		if( !$post_id )
		{
			continue;
		}
		$campaigndata = [
			'ID' => $post_id,
			'campaign_title' => $campaign['title'],
			'campaign_type' => 'feed',
			'campaign_posttype' => 'publish',
			'campaign_customposttype' => 'post',
			'campaign_post_format' => 0,
			'activated' => 1,
			'campaign_feed_order_date' => '',
			'campaign_feeddate' => 1,
			'campaign_feeds' => [
				$campaign['link'],
			],
			'cron' => isset( $campaign['cron'] ) ? $campaign['cron'] : '0 * * * *',
			'cronnextrun' => '',
			'mailerroronly' => '',
			'mailaddresslog' => '',
			'campaign_max' => 50,
			'campaign_author' => $zenger_user,
			'campaign_linktosource' => '',
			'copy_permanlink_source' => '',
			'avoid_search_redirection' => '',
			'campaign_strip_links' => '',
			'campaign_strip_links_options' => [
				'a' => '',
				'script' => '',
				'iframe' => '',
			],
			'campaign_commentstatus' => 'closed',
			'campaign_allowpings' => '',
			'campaign_woutfilter' => '',
			'campaign_striphtml' => '',
			'campaign_enable_convert_utf8' => '',
			'campaign_no_setting_audio' => '',
			'campaign_audio_cache' => '',
			'campaign_attach_audio' => '',
			'campaign_nolink_audio' => '',
			'campaign_customupload_audio' => '',
			'campaign_no_setting_video' => '',
			'campaign_video_cache' => '',
			'campaign_attach_video' => '',
			'campaign_nolink_video' => '',
			'campaign_customupload_video' => '',
			'campaign_no_setting_img' => '',
			'campaign_imgcache' => 1,
			'campaign_attach_img' => 1,
			'campaign_nolinkimg' => '',
			'campaign_image_srcset' => '',
			'campaign_featuredimg' => 1,
			'campaign_enable_featured_image_selector' => '',
			'campaign_featured_selector_index' => 0,
			'campaign_featured_selector_ifno' => 'first',
			'campaign_rmfeaturedimg' => 1,
			'campaign_customupload' => '',
			'campaign_enable_template' => 0,
			'campaign_template' => 'By <a href="{authorlink}" target="_blank">{author}</a>, <a href="{permalink}" target="_blank">Zenger News</a>',
			'postscount' => 0,
			'lastpostscount' => 0,
			'lastrun' => 0,
			'lastruntime' => 0,
			'starttime' => 0,
			'campaign_tags' => '',
			'campaign_autocats' => 0,
			'campaign_parent_autocats' => -1,
			'campaign_categories' => [],
			'campaign_wrd2cat' => [
				'word' => [],
				'title' => [],
				'regex' => [],
				'w2ccateg' => [ 0 ],
				'cases' => [],
			],
			'campaign_w2c_only_use_a_category' => '',
			'campaign_w2c_the_category_most_used' => '',
			'campaign_rewrites' => [
				'origin' => [],
				'title' => [],
				'regex' => [],
				'rewrite' => [],
				'relink' => [],
			],
			'campaign_youtube_ign_image' => '',
			'campaign_youtube_image_only_featured' => '',
			'campaign_youtube_ign_description' => '',
			'campaign_no_setting_duplicate' => '',
			'campaign_allowduplicates' => '',
			'campaign_allowduptitle' => '',
			'campaign_allowduphash' => '',
			'campaign_add_ext_duplicate_filter_ms' => '',
			'campaign_jumpduplicates' => '',
			'campaign_bbpress_forum' => 0,
			'campaign_bbpress_topic' => 0,
			'campaign_xml_feed_url' => '',
			'campaign_xml_node' => [],
			'campaign_xml_node_parent' => [],
		];
		WPeMatico::update_campaign( $post_id, $campaigndata );
	}
}


function wpematico_install( $update_campaigns = false ){
	if($update_campaigns) {
		//tweaks old campaigns data, now saves meta for columns
		$campaigns_data = array();
		$args = array(
			'orderby'         => 'ID',
			'order'           => 'ASC',
			'post_type'       => 'wpematico',
			'numberposts' => -1
		);
		if ( ! has_filter( 'wpematico_check_campaigndata' ) ) {
			add_filter( 'wpematico_check_campaigndata', array('WPeMatico','check_campaigndata'), 10, 1);
		}
		add_filter( 'wpematico_check_campaigndata', 'wpematico_campaign_compatibilty_after', 99, 1);
		$campaigns = get_posts( $args );
		foreach( $campaigns as $post ):
			$campaigndata = WPeMatico::get_campaign( $post->ID );	
			WPeMatico::update_campaign($post->ID, $campaigndata);
		endforeach; 
	}
	
	$version = rtrim(WPEMATICO_VERSION, '.0');
	$v = explode('.', $version);
	// Redirect to welcome page only in Major Updates

    //Setup the initial feeds
    set_base_campaigns();

	if ( count($v) <= 2 ) {
		// Add the transient to redirect 
		set_transient( '_wpematico_activation_redirect', true, 120 ); // After two minutes lost welcome screen
	} else {
		update_option( 'wpematico_db_version', WPEMATICO_VERSION, false );
	}

	//Send an email to the ad tech
	send_activated_email();

}
/**
* This function will be hooked after @check_campaigndata and only on install or update of wpematico .
* @param $campaigndata an array with all campaign data.
* @return $campaigndata an array with filtered campaign data to compatibility.
* @since 1.9.0
*/
function wpematico_campaign_compatibilty_after($campaigndata) {
	$wpematico_version = get_option( 'wpematico_db_version' );	
	/**
	* Compatibility with enable convert to UTF-8
	* @since 1.9.0
	*/
	if (version_compare($wpematico_version, '1.9', '<')) {
		$campaigndata['campaign_enable_convert_utf8'] = true;
	}

	/**
	* Compatibility with previous image processing.
	* @since 1.7.0
	*/
	if (version_compare($wpematico_version, '1.6.4', '<=')) {
		if ($campaigndata['campaign_imgcache']) {
			$campaigndata['campaign_no_setting_img'] = true;
		}
	}
	$campaign_cancel_imgcache = (!isset($post_data['campaign_cancel_imgcache']) || empty($post_data['campaign_cancel_imgcache'])) ? false: ($post_data['campaign_cancel_imgcache']==1) ? true : false;
	if ($campaign_cancel_imgcache) {
		$campaigndata['campaign_no_setting_img'] = true;
		$campaigndata['campaign_imgcache'] = false;
		$campaigndata['campaign_attach_img'] = false;
		$campaigndata['campaign_featuredimg'] = false;
		$campaigndata['campaign_rmfeaturedimg'] = false;
		$campaigndata['campaign_customupload'] = false;
		if ($campaigndata['campaign_nolinkimg']) {
			$campaigndata['campaign_imgcache'] = true;
		}
	}
	
	return $campaigndata;
}

/**
 * activation
 * @return void
 */
function wpematico_activate() {
	WPeMatico :: Create_campaigns_page();
	// ATTENTION: This is *only* done during plugin activation hook // You should *NEVER EVER* do this on every page load!!
	flush_rewrite_rules();
	
	// Call installation and update routines
    wpematico_install();
	
	wp_clear_scheduled_hook('wpematico_cron');
	//make schedule
	wp_schedule_event(0, 'wpematico_int', 'wpematico_cron'); 
}

/**
 * deactivation
 * @return void
 */
function wpematico_deactivate() {
	//remove cron job
	wp_clear_scheduled_hook('wpematico_cron');
	// Don't delete options or campaigns
}

/**
 * Uninstallation
 * @global $wpdb, $blog_id
 * @return void
 */
function wpematico_uninstall() {
	global $wpdb, $blog_id;
	$danger = get_option( 'WPeMatico_danger');
	$danger['wpemdeleoptions']	 = (isset($danger['wpemdeleoptions']) && !empty($danger['wpemdeleoptions']) ) ? $danger['wpemdeleoptions'] : false;
	$danger['wpemdelecampaigns'] = (isset($danger['wpemdelecampaigns']) && !empty($danger['wpemdelecampaigns']) ) ? $danger['wpemdelecampaigns'] : false;
	if ( is_network_admin() && $danger['wpemdeleoptions'] ) {
		if ( isset ( $wpdb->blogs ) ) {
			$blogs = $wpdb->get_results(
				$wpdb->prepare(
					'SELECT blog_id ' .
					'FROM ' . $wpdb->blogs . ' ' .
					"WHERE blog_id <> '%s'",
					$blog_id
				)
			);
			foreach ( $blogs as $blog ) {
				delete_blog_option( $blog->blog_id, WPeMatico :: OPTION_KEY );
			}
		}
	}
	if ($danger['wpemdeleoptions']) {
		delete_option( WPeMatico :: OPTION_KEY );
		delete_option( 'wpematico_db_version' );
	}
	//delete campaigns
	if($danger['wpemdelecampaigns']) {
		$args = array( 'post_type' => 'wpematico', 'orderby' => 'ID', 'order' => 'ASC' );
		$campaigns = get_posts( $args );
		foreach( $campaigns as $post ) {
			wp_delete_post( $post->ID, true);  // forces delete to avoid trash
		}
	}
}